import json
from types import MappingProxyType, SimpleNamespace
from .models import DynamicConfig, VALUE_TYPE_LIST


class Config:
    @staticmethod
    def _string_to_data(value_type: str, value: str):
        assert value_type in VALUE_TYPE_LIST, f"{value_type} type not in {VALUE_TYPE_LIST}"

        if value_type == 'int':
            return int(value)
        if value_type == 'float':
            return float(value)
        if value_type == 'str':
            return str(value)
        if value_type == 'json':
            return json.loads(value)
        if value_type == 'bool':
            return bool(value)

    @classmethod
    def all(cls, is_show: bool = True, is_dict: bool = False):
        return cls._get_config(is_show=is_show, is_dict=is_dict)

    @classmethod
    def _get_config(cls, is_show: bool = True, is_dict: bool = False):
        queryset = DynamicConfig.objects.filter(is_show=is_show)
        config = {item.key: cls._string_to_data(item.value_type, item.value)
                  for item in queryset}
        return config if is_dict else SimpleNamespace(**config)

    def __getattr__(self, item):
        config = self.all(is_dict=True)
        if item not in config:
            raise AttributeError(f"'Config' object has no attribute '{item}'")
        return config[item]



from django.db import models

# Create your models here.

VALUE_TYPE_CHOICES = (
    ("int", "Int"),
    ("float", "Float"),
    ("str", "String"),
    ("bool", "Boolean"),
    ("json", "JSON")
)
VALUE_TYPE_LIST = [_type for _type, _ in VALUE_TYPE_CHOICES]


class DynamicConfig(models.Model):
    key = models.CharField("Key", unique=True, max_length=32)
    value = models.TextField("Value")
    value_type = models.CharField("Value Type", max_length=12, choices=VALUE_TYPE_CHOICES)
    is_show = models.BooleanField("Whether the API returns.", default=True)
    desc = models.CharField("Description about this field", max_length=128, null=True)



# Generated by Django 2.2.4 on 2019-08-14 08:15

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DynamicConfig',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=32, unique=True, verbose_name='Key')),
                ('value', models.TextField(verbose_name='Value')),
                ('value_type', models.CharField(choices=[('int', 'Int'), ('float', 'Float'), ('str', 'String'), ('bool', 'Boolean'), ('json', 'JSON')], max_length=12, verbose_name='Value Type')),
                ('is_show', models.BooleanField(default=True, verbose_name='Whether the API returns.')),
                ('desc', models.CharField(max_length=128, null=True, verbose_name='Description about this field')),
            ],
        ),
    ]



from django.contrib import admin
from .models import DynamicConfig

# Register your models here.


@admin.register(DynamicConfig)
class DynamicConfigAdmin(admin.ModelAdmin):
    list_display = ("id", 'key', 'value', 'value_type', "is_show", "desc")
    list_filter = ('value_type', "is_show")
    search_fields = ('key',)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="django-dynamic-config",
    version='0.0.1',
    author="ansheng",
    author_email='ianshengme@gmail.com',
    description="Django Dynamic Config",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/anshengme/django-dynamic-config",
    keywords=['django', 'django config', 'django dynamic config'],
    packages=setuptools.find_packages(),
    license='LICENSE',
    install_requires=[
        'django'
    ],
    classifiers=[
        'Framework :: Django',
        'Framework :: Django :: 2.0',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'License :: OSI Approved :: MIT License',
        'Environment :: Web Environment',
    ]
)
